@@ -1,6 +1,6 @@
 /*
  * Ncview by David W. Pierce.  A visual netCDF file viewer.
- * Copyright (C) 1993 through 2010 David W. Pierce
+ * Copyright (C) 1993 through 2019 David W. Pierce
  *
  * This program  is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, Version 3, as 
@@ -75,6 +75,8 @@ Stringlist *read_in_state;
 static void init_cmaps_from_data();
 static void init_cmap_from_data( char *colormap_name, int *data );
 static int get_cmaps_from_dir( char *dir_name );
+static int ncview_cmap_suffix( char *s, int *n_suffix );
+static int any_var_in_group( NCVar *var );
 
 /***********************************************************************************************/
 	int
@@ -96,7 +98,7 @@ main( int argc, char **argv )
 		found_state_file = FALSE;
 
 	in_parse_args               ( &argc, argv );
-	input_files = parse_options ( argc,  argv );
+	input_files = parse_options ( argc,  argv );	/* This parses ALL the non-X11 command line options, not just the input files */
 	determine_file_type         ( input_files );
 
 	options.window_title = input_files->string;
@@ -110,9 +112,22 @@ main( int argc, char **argv )
 		exit( -1 );
 		}
 
+	/* If any vars are in groups, we build the interface differently. 
+	 * I pass this information through the global "options" struct.
+	 */
+	if( any_var_in_group( variables )) {
+		options.enable_group_sel = TRUE;
+		options.varsel_style = VARSEL_MENU;
+		}
+	else
+		options.enable_group_sel = FALSE;
+
 	/* This initializes the colormaps, and then the X widows system */
+	if( options.debug ) printf( "Initializing display interface...\n" );
 	initialize_display_interface(); 
+	if( options.debug ) printf( "Initializing printing subsystem...\n" );
 	print_init();
+	if( options.debug ) printf( "Initializing overlays...\n" );
 	overlay_init();
 
 	/* If there is only one variable, make it the active one */
@@ -204,6 +219,9 @@ parse_options( int argc, char *argv[] )
 			else if( strncmp( argv[i], "-beep", 5 ) == 0 )
 				options.beep_on_restart = TRUE;
 
+			else if( strncmp( argv[i], "-pause_on_restart", 17 ) == 0 )
+				options.stop_on_restart = TRUE;
+
 			else if( strncmp( argv[i], "-fra", 4 ) == 0 )
 				options.dump_frames = TRUE;
 
@@ -236,6 +254,9 @@ parse_options( int argc, char *argv[] )
 			else if( strncmp( argv[i], "-shrink_mode", 12) == 0 )
 				options.shrink_method = SHRINK_METHOD_MODE;
 
+			else if( strncmp( argv[i], "-repl", 5) == 0 )
+				options.blowup = BLOWUP_REPLICATE;
+
 			else if( strncmp( argv[i], "-c", 2 ) == 0 ) {
 				print_copying();
 				exit( 0 );
@@ -258,6 +279,15 @@ parse_options( int argc, char *argv[] )
 				i++;
 				}
 
+			else if( strncmp( argv[i], "-missvalrgb", 11 ) == 0 ) {
+				sscanf( argv[i+1], "%d", &(options.missval_r) );
+				i++;
+				sscanf( argv[i+1], "%d", &(options.missval_g) );
+				i++;
+				sscanf( argv[i+1], "%d", &(options.missval_b) );
+				i++;
+				}
+
 			else if( strncmp( argv[i], "-nc", 3 ) == 0 ) {
 				sscanf( argv[i+1], "%d", &(options.n_colors) );
 				if( options.n_colors > 255 ) {
@@ -344,6 +374,7 @@ initialize_misc()
 	options.show_sel	 = FALSE;
 	options.want_extra_info  = FALSE;
 	options.beep_on_restart  = FALSE;
+	options.stop_on_restart  = FALSE;
 	options.small  		 = FALSE;
 	options.blowup_type      = DEFAULT_BLOWUP_TYPE;
 	options.save_frames      = DEFAULT_SAVEFRAMES;
@@ -363,6 +394,11 @@ initialize_misc()
 
 	options.maxsize_pct	 = 75;	/* maximum size of a window, in percent of screen, before switching to scrollbars */
 
+	/* Set default color to use for missing data */
+	options.missval_r 	= 255;
+	options.missval_g 	= 255;
+	options.missval_b 	= 255;
+
 	framestore.frame = NULL;
 	framestore.valid = FALSE;
 
@@ -408,13 +444,45 @@ initialize_colormaps()
 	n_colormaps += get_cmaps_from_dir( "." );
 }
 
+/***********************************************************************************************/
+/* Returns 0 if the passed char string "s" has a valid ncview cmap file suffix, and 1 otherwise.
+ * If a 0 is returned, then n_suffix is also set to the suffix length, including the period.
+ */
+	int
+ncview_cmap_suffix( char *s, int *n_suffix )
+{
+	int		nc;
+
+	if( s == NULL ) 
+		return( 1 );
+
+	nc = strlen( s );
+	if( nc < 5 ) 
+		return( 1 );
+
+	if( strncasecmp( (s+nc-4), ".ncm", 4 ) == 0 ) {
+		*n_suffix = 4;
+		return( 0 );
+		}
+
+	if( nc < 7 ) 
+		return( 1 );
+
+	if( strncasecmp( (s+nc-6), ".ncmap", 6 ) == 0 ) {
+		*n_suffix = 6;
+		return( 0 );
+		}
+
+	return( 1 );
+}
+
 /***********************************************************************************************/
 	int
 get_cmaps_from_dir( char *dir_name )
 {
 	DIR		*ncdir = NULL;
 	struct dirent	*dir_entry;
-	int		n_colormaps = 0;
+	int		n_colormaps = 0, n_suffix;
 
 	if( options.debug ) 
 		fprintf( stderr, "Getting colormaps from dir >%s<\n", dir_name );
@@ -427,10 +495,8 @@ get_cmaps_from_dir( char *dir_name )
 		/* Additional allowed filenames as per suggestion by 
 		 * Arlindo da Silva for his Windows port
 		 */
-		if( (strstr( dir_entry->d_name, ".NCM"   ) != NULL) ||
-		    (strstr( dir_entry->d_name, ".ncm"   ) != NULL) ||
-		    (strstr( dir_entry->d_name, ".ncmap" ) != NULL) ) {
-			init_cmap_from_file( dir_name, dir_entry->d_name );
+		if( ncview_cmap_suffix( dir_entry->d_name, &n_suffix ) == 0) {
+			init_cmap_from_file( dir_name, dir_entry->d_name, n_suffix );
 			n_colormaps++;
 			}
 		dir_entry = readdir( ncdir );
@@ -495,7 +561,7 @@ init_cmap_from_data( char *colormap_name, int *data )
 
 /***********************************************************************************************/
 	void
-init_cmap_from_file( char *dir_name, char *file_name )
+init_cmap_from_file( char *dir_name, char *file_name, int n_suffix )
 {
 	char 	*colormap_name;
 	FILE	*cmap_file;
@@ -507,10 +573,10 @@ init_cmap_from_file( char *dir_name, char *file_name )
 	if( options.debug ) 
 		fprintf( stderr, "    ... initting cmap >%s<\n", file_name );
 
-	/* Colormap name is the file name without the '.ncmap' extension */
-	colormap_name = (char *)malloc( strlen(file_name)-5 );
-	strncpy( colormap_name, file_name, strlen(file_name)-6 );
-	*(colormap_name + strlen(file_name)-6) = '\0';
+	/* Colormap name is the file name without the '.ncmap' or '.ncm' extension */
+	colormap_name = (char *)malloc( strlen(file_name)-(n_suffix-1) );
+	strncpy( colormap_name, file_name, strlen(file_name)-n_suffix );
+	*(colormap_name + strlen(file_name)-n_suffix) = '\0';
 
 	/* Make sure this colormap name isn't already known */
 	if( x_seen_colormap_name( colormap_name )) {
@@ -534,8 +600,8 @@ init_cmap_from_file( char *dir_name, char *file_name )
 		if( fgets( line, 128, cmap_file ) == NULL ) {
 			fprintf( stderr, "ncview: init_cmap_from_file: file %s finished ",
 					long_file_name );
-			fprintf( stderr, "on line %d, but should have 256 lines\n", i+1 );
-			exit( -1 );
+			fprintf( stderr, "on line %d, but should have 256 lines: not a valid ncview cmap file\n", i+1 );
+			return;
 			}
 
 		nentries = sscanf( line, "%d %d %d", &r_entry, &g_entry, &b_entry );
@@ -580,9 +646,12 @@ init_cmap_from_file( char *dir_name, char *file_name )
 	void
 initialize_file_interface( Stringlist *input_files )
 {
-	int	idim, nvars, nfiles;
+	int	i, idim, nvars, nfiles;
 	NCVar	*var;
 
+	if( options.debug ) 
+		fprintf( stderr, "Initializing file interface...\n" );
+
 	nfiles = stringlist_len( input_files );
 
 	while( input_files != NULL ) {
@@ -590,7 +659,7 @@ initialize_file_interface( Stringlist *input_files )
 		input_files = input_files->next;
 		}
 	if( options.debug ) 
-		fprintf( stderr, "Calculating dim min & maxes...\n" );
+		fprintf( stderr, "...calculating dim min & maxes...\n" );
 	calc_dim_minmaxes();
 
 	/* Get the effective dimensionality of all the vars.
@@ -602,12 +671,18 @@ initialize_file_interface( Stringlist *input_files )
 	while( var != NULL ) {
 		nvars++;
 		var->effective_dimensionality = 0;
-		for( idim=0; idim<var->n_dims; idim++ ) 
+		for( idim=0; idim<var->n_dims; idim++ ) {
 			if( *(var->size + idim) > 1 )
 				var->effective_dimensionality++;
-		if( options.debug ) 
+			if( options.debug ) 
+				fprintf( stderr, "var %s has %d dims, dim %d: >%s< len %ld\n",
+					var->name, var->n_dims, idim, 
+					var->dim[idim]->name, var->dim[idim]->size );
+			}
+		if( options.debug ) {
 			fprintf( stderr, "variable %s had effective_dimensionality of %d\n",
 				var->name, var->effective_dimensionality );
+			}
 		var = var->next;
 		}
 
@@ -619,6 +694,9 @@ initialize_file_interface( Stringlist *input_files )
 
 	if( nvars > options.listsel_max )
 		options.varsel_style = VARSEL_MENU;
+
+	if( options.debug ) 
+		fprintf( stderr, "Done initializing file interface...\n" );
 }
 
 /***********************************************************************************************/
@@ -632,7 +710,9 @@ initialize_display_interface()
 	 */
 	x_check_legal_colormap_loaded();
 
+	if( options.debug ) printf( "...initializing X interface\n" );
 	in_initialize();
+	if( options.debug ) printf( "...done with initializing X interface\n" );
 }
 
 /***********************************************************************************************/
@@ -678,6 +758,21 @@ create_default_colormap()
 	in_create_colormap( "default", r, g, b );
 }
 
+/***********************************************************************************************/
+int any_var_in_group( NCVar *var ) {
+
+	NCVar	*cursor;
+
+	cursor = var;
+	while( cursor != NULL ) {
+		if( count_nslashes( cursor->name ) > 0 ) 
+			return( 1 );
+		cursor = cursor->next;
+		}
+
+	return( 0 );
+}
+
 /***********************************************************************************************/
 	void
 useage()
@@ -694,10 +789,12 @@ fprintf( stderr, "		(\"-minmax slow\"), or all entries (\"-minmax all\").\n" );
 fprintf( stderr, "	-frames: Dump out PNG images (to make a movie, for instance)\n" );
 fprintf( stderr, "	-nc: 	Specify number of colors to use.\n" );
 fprintf( stderr, "	-no1d: 	Do NOT allow 1-D variables to be displayed.\n" );
+fprintf( stderr, "	-repl: 	Set default blowup type to replicate rathern than bilinear.\n" );
 fprintf( stderr, "	-calendar: Specify time calendar to use, overriding value in file. Known: noleap standard gregorian 365_day 360_day.\n" );
 fprintf( stderr, "	-private: Use a private colormap.\n" );
 fprintf( stderr, "	-debug: Print lots of debugging info.\n" );
 fprintf( stderr, "	-beep: 	Ring the bell when the movie restarts at frame zero.\n" );
+fprintf( stderr, "	-pause_on_restart: Pause playback when the movie restarts at frame zero.\n" );
 fprintf( stderr, "	-extra: Put some extra information on the display window.\n" );
 fprintf( stderr, "	-mtitle: My title to use on the display window.\n" );
 fprintf( stderr, "	-noautoflip: Do not automatically flip image, even\n" );
@@ -710,6 +807,7 @@ fprintf( stderr, "	-listsel_max NN: max number of vars allowed before switching
 fprintf( stderr, "	-no_color_ndims: do NOT color the var selection buttons by their dimensionality\n" );
 fprintf( stderr, "	-no_auto_overlay: do NOT automatically put on continental overlays\n" );
 fprintf( stderr, "	-autoscale: scale color map of EACH frame to range of that frame. Note: MUCH SLOWER!\n" );
+fprintf( stderr, "	-missvalrgb RRR GGG BBB: specifies 3 integers (range: 0 to 255) to use for missing value color\n" );
 fprintf( stderr, "	-maxsize: specifies max size of window before scrollbars are added. Either a single\n" );
 fprintf( stderr, "              integer between 30 and 100 giving percentage, or two integers separated by a\n" );
 fprintf( stderr, "              comma giving width and height. Ex: -maxsize 75  or -maxsize 800,600\n" );
@@ -728,7 +826,7 @@ print_disclaimer()
 {
 fprintf( stderr, "%s\n", PROGRAM_ID );
 fprintf( stderr, "http://meteora.ucsd.edu:80/~pierce/ncview_home_page.html\n" );
-fprintf( stderr, "Copyright (C) 1993 through 2011, David W. Pierce\n" );
+fprintf( stderr, "Copyright (C) 1993 through 2019, David W. Pierce\n" );
 fprintf( stderr, "Ncview comes with ABSOLUTELY NO WARRANTY; for details type `ncview -w'.\n" );
 fprintf( stderr, "This is free software licensed under the Gnu General Public License version 3; type `ncview -c' for redistribution details.\n\n" );
 }
@@ -737,7 +835,7 @@ fprintf( stderr, "This is free software licensed under the Gnu General Public Li
 	void
 print_no_warranty()
 {
-printf( "\n The program `ncview' is Copyright (C) 1993 through 2010 David W. Pierce, and\n" );
+printf( "\n The program `ncview' is Copyright (C) 1993 through 2019 David W. Pierce, and\n" );
 printf( "is subject to the terms and conditions of the Gnu General Public License,\n" );
 printf( "Version 3. For information on copying, modifying, or distributing `ncview',\n" );
 printf( "type `ncview -c'.\n" );
@@ -777,7 +875,7 @@ printf( "POSSIBILITY OF SUCH DAMAGES.\n" );
 	void 
 print_copying()
 {
-printf( "  The program `ncview' is Copyright (C) 1993 through 2010, David W. Pierce, and \n" );
+printf( "  The program `ncview' is Copyright (C) 1993 through 2019, David W. Pierce, and \n" );
 printf( "is subject to the terms and conditions of the Gnu General Public License,\n" );
 printf( "Version 3.  Ncview comes with NO WARRANTY; for further information, type\n" );
 printf( "`ncview -w'.\n" );
