@@ -1,6 +1,6 @@
 /*
  * Ncview by David W. Pierce.  A visual netCDF file viewer.
- * Copyright (C) 1993 through 2011 by David W. Pierce
+ * Copyright (C) 1993 through 2013 by David W. Pierce
  *
  * This program  is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as 
@@ -31,8 +31,8 @@
 #include <udunits2.h>
 #endif
 
-#define PROGRAM_ID		"Ncview 2.1.1 David W. Pierce  1 Aug 2011"
-#define PROGRAM_VERSION_STRING	"2.1.1"
+#define PROGRAM_ID		"Ncview 2.1.9 David W. Pierce  14 July 2019"
+#define PROGRAM_VERSION_STRING	"2.1.9"
 #define APP_RES_VERSION 	1.93
 
 #ifndef TRUE
@@ -130,15 +130,15 @@
 
 /*****************************************************************************/
 /* Maximum name length of a variable */
-#define MAX_VAR_NAME_LEN	132
+#define MAX_VAR_NAME_LEN	4095
 
 /*****************************************************************************/
 /* Maximum name length of a file */
-#define MAX_FILE_NAME_LEN	1024
+#define MAX_FILE_NAME_LEN	4095
 
 /*****************************************************************************/
 /* Maximum name length of a recdim units */
-#define MAX_RECDIM_UNITS_LEN	1024
+#define MAX_RECDIM_UNITS_LEN	4095
 
 /*****************************************************************************/
 /* Possible interpretations for the change_view routine; either change
@@ -399,6 +399,10 @@ typedef struct {
 					 	* are global, rather than local to
 					 	* a file.
 					 	*/
+	int	user_set_blowup;		/* Initializes to -99999, then saves user-specified
+	  					 * value of 'blowup' for this var so it can be
+						 * used again if we leave this var & then come back
+						 */
 	int	auto_set_no_range;		/* '1' if we autoset a range of -1,1 based
 						 * on not having a valid range for this var
 						 */
@@ -512,6 +516,7 @@ typedef struct {
 		listsel_max,	/* if # of vars is more than this, auto switch from VARSEL_LIST to VARSEL_MENU */
 		color_by_ndims,	/* if 1, then button is color coded by # of effective dims */
 		beep_on_restart,
+		stop_on_restart,
 		auto_overlay,	/* if 1, then tries to figure out if coastlines should automatically be added */
 		blowup,
 		maxsize_pct,	/* -1 if a width/height pair specified instead */
@@ -532,6 +537,10 @@ typedef struct {
 	int	save_frames;	/* If true, try to save frames in core for faster display */
 	float	frame_delay;	/* Normalied to be between 0.0 and 1.0 */
 
+	int	enable_group_sel;	/* TRUE if we have some vars in groups, so interface must incl. grp selection */
+
+	int	missval_r, missval_g, missval_b;	/* 0-255 values of R, G, B for missing data */
+
 	OverlayOptions *overlay;
 } Options;
 
